/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAS code for implementing the DK selection model and the local influence under the DK selection model. 					%
%																															%
% These models were applied to CD4 count measurement data (IMPI_29_2016) from the IMPI trial 								%
% 																															%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

proc mixed DATA=IMPI_29_2016; /* Data name is "IMPI_29_2016"*/
model sqcd4 = time pred pred*time art pred*art transage / DDFM=KR S;
run;

/*************************************/
/* (1) DIGGLE-KENWARD MODEL : MCAR   */
/*************************************/

proc iml;
use IMPI_2016;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
	beta= 24.3408//0.4241//0.2943//0.02986//1.9891//-0.05294//-3.2065;
	sigma11=27.5557;
  	sigma21=23.1566;
 	sigma22=29.7099;
 	sigma31=21.7018;
 	sigma32=22.8297;
 	sigma33=29.6190;
 	sigma41=17.7755;
 	sigma42=17.6218;
 	sigma43=20.0637;
 	sigma44=25.7221;
	sigma51 = 18.4084;
	sigma52 = 18.7202;
	sigma53 = 20.3257;
	sigma54 = 22.7242;
	sigma55 = 26.2859;
	v = j(5,5,1);
	v[1,1] = sigma11;
	v[1,2] = sigma21;
	v[2,1] = sigma21;
	v[2,2] = sigma22;
	v[1,3] = sigma31;
	v[3,1] = sigma31;
	v[2,3] = sigma32;
	v[3,2] = sigma32;
	v[3,3] = sigma33;
	v[1,4] = sigma41;
	v[4,1] = sigma41;
	v[2,4] = sigma42;
	v[4,2] = sigma42;
	v[3,4] = sigma43;
	v[4,3] = sigma43;
	v[4,4] = sigma44;
	v[5,1] = sigma51;
	v[1,5] = sigma51;
	v[5,2] = sigma52;
	v[2,5] = sigma52;
	v[5,3] = sigma53;
	v[3,5] = sigma53;
	v[5,4] = sigma54;
	v[4,5] = sigma54;
	v[5,5] = sigma55;
	u=root(v);
	v2=t(u)*u;
	uvec=colvec(u);
	nozero=loc(uvec<>0);
	uvec=uvec[nozero];
	psi= 1;
	initial=beta//uvec//psi;	
create initial var {initial};
append;
	nsub=137;
	ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;


proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1]=parameters[23];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,23,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;

data impi2aMCARcovar;
set covmatrix;
run;
proc print data=impi2aMCARcovar;
run;

data impi2aMCAR;
set result;
run;
proc print data=impi2aMCAR;
run;

data impi2aMCARhessian;
set hessianmatrix;
run;
proc print data=impi2aMCARhessian;
run;
data impi2aMCARloglik;
set loglik;
run;
proc print data=impi2aMCARloglik;
run;

/*****************************************/
/* (2)  DIGGLE-KENWARD MODEL UNDER MAR   */
/*****************************************/
proc iml;
use IMPI_2016;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
use impi2aMCAR;
read all var{est} into mcarest;
*initial=mcarest//0.5;
initial=mcarest//0.05;
create initial var {initial};
append;
nsub=137;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;
	
proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:2]=parameters[23:24];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

   if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,24,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
data impi2aMARcovar;
set covmatrix;
run;
proc print data=impi2aMARcovar;
run;

data impi2aMAR;
set result;
run;
proc print data=impi2aMAR;
run;

data impi2aMARhessian;
set hessianmatrix;
run;
proc print data=impi2aMARhessian;
run;
data impi2aMARloglik;
set loglik;
run;
proc print data=impi2aMARloglik;
run;


/*************************************/
/* (3) DIGGLE-KENWARD MODEL : MNAR   */
/*************************************/
proc iml;
use IMPI_2016;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
use impi2aMAR;
read all var{est} into mcarest;
*initial=mcarest//0.5;
initial=mcarest//0.05;
create initial var {initial};
append;
nsub=137;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;
proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:3]=parameters[23:25];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

       
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,25,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
data impi2aNMARcovar;
set covmatrix;
run;
proc print data=impi2aNMARcovar;
run;

data impi2aNMAR;
set result;
run;
proc print data=impi2aNMAR;
run;

data impi2aNMARhessian;
set hessianmatrix;
run;
proc print data=impi2aNMARhessian;
run;
data impi2aNMARloglik;
set loglik;
run;
proc print data=impi2aNMARloglik;
run;



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%/
' (4) LOCAL INFLUENCE	
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

proc iml;
use IMPI_29_2016;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aMCAR;
read all var{est} into mcarest;
*initial=mcarest//0.5;
initial=mcarest//0.05;
create initial var {initial};
append;
nsub=137;
ntime=4;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u22=parameters[12];
	u23=parameters[13];
	u24=parameters[14];
	u33=parameters[15];
	u34=parameters[16];
	u44=parameters[17];
	psi=j(3,1,0);
	psi[1:2]=parameters[18:19];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

   if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;


start delta(parameters) global(lastobs,vcurr,ecurr,x,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u22=parameters[12];
	u23=parameters[13];
	u24=parameters[14];
	u33=parameters[15];
	u34=parameters[16];
	u44=parameters[17];
	psi[1:2]=parameters[18:19];	nbeta=nrow(beta);
	nsigma=10;
	npsi=2;
	ind=1;
	ll=0;
	ind=1;

	do while (ind<=nsub);
     /* select information on one particular patient */
  		xi = x[(ind-1)*ntime+1:ind*ntime,];
  		yi = y[(ind-1)*ntime+1:ind*ntime];
  		mui = xi*beta;
		ui = j(ntime,ntime,1);
		ui[1,1] = u11;
		ui[1,2] = u12;
		ui[1,3] = u13;
		ui[1,4] = u14;
		ui[2,1] = 0;
		ui[2,2] = u22;
		ui[2,3] = u23;
		ui[2,4] = u24;
		ui[3,1] = 0;
		ui[3,2] = 0;
		ui[3,3] = u33;
		ui[3,4] = u34;
		ui[4,1] = 0;
		ui[4,2] = 0;
		ui[4,3] = 0;
		ui[4,4] = u44;
		vi=t(ui)*ui;
		sigma1=vi[1,1];
		sigma12=vi[1,2];
		sigma13=vi[1,3];
		sigma14=vi[1,4];
		sigma2=vi[2,2];
		sigma23=vi[2,3];
		sigma24=vi[2,4];
		sigma3=vi[3,3];
		sigma34=vi[3,4];
		sigma4=vi[4,4];
		psi=j(3,1,0);
		pi = constant('PI');
		g=j(nsub,1,0);

		/* yobs and dropout */

  		if (yi[nrow(yi)]=.) then do;                     
        	dropout=1;
			j=nrow(yi);
        	do while (yi[j-1]=.);
          		j=j-1;
        	end;
        	di=j;
			yobs=yi[1:di-1];
      	end;
  		else do;                                                  
        	dropout=0;
        	yobs=yi;
    	end;
 
 		sequence = t(1:nrow(yobs)); 
		miss=yobs||sequence; 
  		i=1;
  		do while (i<=nrow(miss));
    		if (miss[i,1]=.) then do;
          		nr=nrow(miss);
        		row=(i-1)*2+1//(i-1)*2+2;
            	miss=remove(miss,row);
            	miss=shape(miss,nr-1,2);
      		end;
    		else 
      			i=i+1;
		end;
  		yobs = miss[,1];
  		nummi = miss[,2];

  			/* calculation of delta  */

  		contribd = j(1,npsi,0);
  		contribm = j(nbeta+nsigma,1,0);

  		if dropout=0 then do;  /* COMPLETER */
      		j=2;
			do while (j <= nrow(yobs));
        		g = exp(psi[1]+yobs[j-1]*psi[2]);
        		g = g/(g+1);
        		hij = 1||yobs[j-1];
        		contribd = contribd-yobs[j]#hij#g#(1-g);
        		j=j+1;
      		end;
    	end;
  		else do;         /* INCOMPLETER */
      		if (di=2) then do;
				hij=yobs;
				g = exp(psi[1]+yobs[nrow(yobs)]*psi[2]);
				g = g/(g+1);
      			muc = mui[di];
      			mup = mui[nummi];
      			hist = yobs;
      			vi11 = vi[nummi,nummi];
      			vi21 = vi[di,nummi];
      			vi11i = inv(vi11);
      			ecurr = muc + vi21*(vi11i)*(hist-mup);
      			contribd = contribd - ecurr#hij#g#(1-g);
      				/* use derivative wrt beta */
      			contribm[1:nbeta] = (1-g)#(xi[di,]-vi21*vi11i*xi[nummi,])`;
      				/* use derivative wrt sigma1 */
      			contribm[nbeta+1] = (1-g)#(0-vi21*vi11i*2*sigma1)*vi11i*(hist-mup);
      				/* use derivative wrt sigma12 */
      			contribm[nbeta+2] = (1-g)#(1-vi21*vi11i*0)*vi11i*(hist-mup);
      				/* use derivative wrt sigma13 */
      			contribm[nbeta+3] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma14 */
      			contribm[nbeta+4] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma2 */
      			contribm[nbeta+5] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma23 */
      			contribm[nbeta+6] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma24 */
				contribm[nbeta+7] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma3 */
				contribm[nbeta+8] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma34 */
				contribm[nbeta+9] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma4 */
				contribm[nbeta+10] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
        	end;
      		else if (di=3) then do;
          		j=2;
          		do while (j <= nrow(yobs));
            		g = exp(psi[1]+yobs[j-1]*psi[2]);
            		g = g/(g+1);
            		hij = 1||yobs[j-1];
            		contribd = contribd-yobs[j]#hij#g#(1-g);
            		j = j+1;
				end;
				g = exp(psi[1]+yobs[nrow(yobs)]*psi[2]);
				g = g/(g+1);
      			muc = mui[di];
      			mup = mui[nummi];
      			hist = yobs;
      			vi11 = vi[nummi,nummi];
      			vi21 = vi[di,nummi];
      			vi11i = inv(vi11);
      			ecurr = muc + vi21*(vi11i)*(hist-mup);
      			contribd = contribd - ecurr#hij#g#(1-g);
      				/* use derivative wrt beta */
      			contribm[1:nbeta] = (1-g)#(xi[di,]-vi21*vi11i*xi[nummi,])`;
      				/* use derivative wrt sigma1 */
				vi21d=0||0;
				vi21d = vi21d[nummi]`;
				helpvector=2*sigma1||0||0||0;
				vi11d=shape(helpvector,2,2);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+1] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma12 */
				vi21d=0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||1||1||0;
				vi11d=shape(helpvector,2,2);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+2] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma13 */
				vi21d=1||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0;
				vi11d=shape(helpvector,2,2);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+3] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma14 */
      			contribm[nbeta+4] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma2 */
				vi21d=0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||2*sigma2;
				vi11d=shape(helpvector,2,2);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+5] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
      				/* use derivative wrt sigma23 */
				vi21d=0||1;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0;
				vi11d=shape(helpvector,2,2);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+6] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma24 */
      			contribm[nbeta+7] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma3 */
      			contribm[nbeta+8] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma34 */
      			contribm[nbeta+9] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma4 */
      			contribm[nbeta+10] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
        	end;
			else if (di=4) then do;
          		j=2;
          		do while (j <= nrow(yobs));
            		g = exp(psi[1]+yobs[j-1]*psi[2]);
            		g = g/(g+1);
            		hij = 1||yobs[j-1];
            		contribd = contribd-yobs[j]#hij#g#(1-g);
            		j = j+1;
				end;
				g = exp(psi[1]+yobs[nrow(yobs)]*psi[2]);
				g = g/(g+1);
      			muc = mui[di];
      			mup = mui[nummi];
      			hist = yobs;
      			vi11 = vi[nummi,nummi];
      			vi21 = vi[di,nummi];
      			vi11i = inv(vi11);
      			ecurr = muc + vi21*(vi11i)*(hist-mup);
      			contribd = contribd - ecurr#hij#g#(1-g);
					/* use derivative wrt beta */
      			contribm[1:nbeta] = (1-g)#(xi[di,]-vi21*vi11i*xi[nummi,])`;
      				/* use derivative wrt sigma1 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=2*sigma1||0||0||0||0||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+1] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma12 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||1||0||1||0||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+2] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma13 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||1||0||0||0||1||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+3] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma14 */
				vi21d=1||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||0||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+4] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma2 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||2*sigma2||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+5] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
      				/* use derivative wrt sigma23 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||0||1||0||1||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+6] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma24 */
				vi21d=0||1||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||0||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+7] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma3 */
				vi21d=0||0||0;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||0||0||0||0||2*sigma3;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+8] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
					/* use derivative wrt sigma34 */
      			vi21d=0||0||1;
				vi21d = vi21d[nummi]`;
				helpvector=0||0||0||0||0||0||0||0||0;
				vi11d=shape(helpvector,3,3);
				vi11d = vi11d[nummi,nummi];
      			contribm[nbeta+9] = (1-g)#(vi21d-vi21*vi11i*vi11d)*vi11i*(hist-mup);
					/* use derivative wrt sigma4 */
      			contribm[nbeta+10] = (1-g)#(0-vi21*vi11i*0)*vi11i*(hist-mup);
        	end;
		end;
  		if (ind=1) then do;
      		inflvec = contribm //(contribd`);
    	end;
  		else do;
      		inflvecind = contribm//(contribd`);
      		inflvec = inflvec||inflvecind;
    	end;
  		ind = ind+1;
	end;
	delta = inflvec;
	return(delta);
finish delta;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,19,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);

/* calculation hessian */

call nlpfdd(maxlik,grad,hessian,"loglik",est);
create result var {est stde};
append;
delta=delta(est);
nbeta=7;
nsigma=10;
npsi=2;
hessianm = hessian[1:(nbeta+nsigma),1:(nbeta+nsigma)];
hessiand = hessian[(nbeta+nsigma+1):(nbeta+nsigma+npsi),(nbeta+nsigma+1):(nbeta+nsigma+npsi)];

/* calculation the C-matrix */

tweedea = j(nbeta+nsigma,nbeta+nsigma,0);
tweedeb = j(nbeta+nsigma,nbeta+nsigma,0);
tweedea[(nbeta+1):(nbeta+nsigma),(nbeta+1):(nbeta+nsigma)]= inv(hessian[(nbeta+1):(nbeta+nsigma),(nbeta+1):(nbeta+nsigma)]);
tweedeb[1:nbeta,1:nbeta] = inv(hessian[1:nbeta,1:nbeta]);
c = 2*delta`*inv(hessian)*delta;
c1 = 2*delta[1:(nbeta+nsigma),]`*(inv(hessian[1:(nbeta+nsigma),1:(nbeta+nsigma)])-tweedea)* delta[1:(nbeta+nsigma),];
c2 = 2*delta[1:(nbeta+nsigma),]`*(inv(hessian[1:(nbeta+nsigma),1:(nbeta+nsigma)])-tweedeb)*delta[1:(nbeta+nsigma),];
call eigen(l,v,-c);
cmax = l[1];
print cmax;
hmax = v[,1];
c3 = 2*delta[(nbeta+nsigma+1):(nbeta+nsigma+npsi),]`*inv(hessiand)*delta[(nbeta+nsigma+1):(nbeta+nsigma+npsi),];
c12 = c-c3;
ci = -vecdiag(c);
c12i = -vecdiag(c12);
c3i = -vecdiag(c3);
c1i = -vecdiag(c1);
c2i = -vecdiag(c2);
subject = 1:nrow(ci);
create c_matrix var {subject ci c12i c3i c1i c2i hmax};
append;
quit;
proc print data=c_matrix;
run;

data MARCmatrix;
set c_matrix;
run;
proc print data=MARCmatrix;
run;
proc means data=MARCmatrix p95 p99 max min;
var ci c1i c2i c12i c3i hmax; 
run;

proc print data=MARCmatrix;
where ci>62.6967641;
run;

proc print data=c_matrix;
where c12i>7;
run;

proc print data=c_matrix;
where c12i>2000;
run;

proc print data=c_matrix;
where c2i>2000;
run;

proc print data=c_matrix;
where c1i>2.85;
run;
 
proc print data=c_matrix;
where abs(hmax)>0.25;
run;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'     FIGURES     
/%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/* Set the graphics environment */
goptions reset=all cback=white width = 5 htitle=10pt htext=12pt; 
/* Use the POINTLABEL option to label the plot points */

proc gplot data=MARCmatrix;
	plot ci*subject;
	symbol1 pointlabel = ("#subject" h=3 font=swiss)interpol=join  value=dot color=black;
	title h=1 'Total C_i';
run;
quit; 

goptions reset=all cback=white htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot c12i*subject/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=3 font=swiss) interpol=join value=dot color=blue;
	title h=1 'C_i(theta)';
run;
quit; 
goptions reset=all cback=white width = 5  htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot c3i*subject/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=3 font=swiss) interpol=join value=dot color=blue;
	title h=2 'C_i(psi)';
run;
quit; 

goptions reset=all cback=white border htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot c1i*subject/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=3 font=swiss) interpol=join value=dot color=blue;
	title h=1 'C_i(beta)';
run;
quit; 
goptions reset=all cback=white htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot c2i*subject/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=3 font=swiss) interpol=join value=dot color=blue;
	title h=1 'C_i(alpha)';
run;
quit; 
goptions reset=all cback=white width = 5 border htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot hmax*subject/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=3 font=swiss) interpol=join value=dot color=blue;
	title h=1 'hmax';
run;
quit; 

goptions reset=all cback=white border htitle=12pt htext=10pt; 
/* Use the POINTLABEL option to label the plot points */
proc gplot data=MARCmatrix;
	plot c2i*c1i/ haxis=axis1 vaxis=axis2;
	symbol1 pointlabel = ("#subject" h=2 font=swiss) interpol=join value=dot color=blue;
	title h=1 'C_i(alpha) vs C_i(beta)';
run;
quit; 

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'
' Global influence analyses
'
'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%/
'
' Analysis of subset 1
'
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/



/***************************************/
/* (5a) MCAR analysis without subset 1 */
/***************************************/

data data1mnarsubset1;
set IMPI_2016;
if id in (94, 93) then delete;
run;
proc iml;
use data1mnarsubset1;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
use impi2aMCAR;
read all var {est} into mcarest;
initial=mcarest;	
create initial var {initial};
append;
nsub=137-2;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;


proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1]=parameters[23];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,23,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;

data MCARsubset1;
set result;
run;
proc print data=MCARsubset1;
run;
data MCARmxlik;
set loglik;
run;
proc print data=MCARmxlik;
run;

/***************************************/
/* (5b) MAR analysis without subset 1  */
/***************************************/
data data1marsubset1;
set IMPI_2016;
if id in (94, 93) then delete;
run;
proc iml;
use data1marsubset1;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aMAR;
read all var{est} into MARrest;
initial=MARrest;
create initial var {initial};
append;
nsub=137-2;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:2]=parameters[23:24];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,24,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;
data MARsubsets1;
set result;
run;
proc print data=MARsubsets1;
run;

data MARmaxlik;
set loglik;
run;
proc print data=MARmaxlik;
run;

/*************************************/
/* (3) DIGGLE-KENWARD MODEL : MNAR   */
/*************************************/
data data1nmarsubset1;
set IMPI_2016;
if id in (94, 93) then delete;
run;
proc iml;
use data1nmarsubset1;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aNMAR;
read all var{est} into NMARrest;
*initial=mcarest//0.5;
initial=NMARrest;
create initial var {initial};
append;
nsub=137-2;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:3]=parameters[23:25];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,25,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
data NMARsubset1;
set result;
run;
proc print data=NMARsubset1;
run;
data NMARmxlik;
set loglik;
run;
proc print data=NMARmxlik;
run;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'
'Analysis of subset 
'
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/***************************************/
/* (5a) MCAR analysis without subset 2 */
/***************************************/


data data1mnarsubset2;
set IMPI_2016;
if id in (33, 78, 133) then delete;
run;
proc iml;
use data1mnarsubset2;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
use impi2aMCAR;
read all var {est} into mcarest;
initial=mcarest;	
create initial var {initial};
append;
nsub=137-3;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;


proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1]=parameters[23];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,23,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;

data MCARsubset2;
set result;
run;
proc print data=MCARsubset2;
run;
data MCARmxlik;
set loglik;
run;
proc print data=MCARmxlik;
run;

/***************************************/
/* (5b) MAR analysis without subset 1  */
/***************************************/
data data1marsubset2;
set IMPI_2016;
if id in (33, 78, 133) then delete;
run;
proc iml;
use data1marsubset2;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aMAR;
read all var{est} into MARrest;
initial=MARrest;
create initial var {initial};
append;
nsub=137-3;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:2]=parameters[23:24];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,24,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;
data MARsubsets2;
set result;
run;
proc print data=MARsubsets2;
run;

data MARmaxlik;
set loglik;
run;
proc print data=MARmaxlik;
run;

/*************************************/
/* (3) DIGGLE-KENWARD MODEL : MNAR   */
/*************************************/
data data2nmarsubset2;
set IMPI_2016;
if id in (33, 78) then delete;
run;
proc iml;
use data2nmarsubset2;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aNMAR;
read all var{est} into NMARrest;
*initial=mcarest//0.5;
initial=NMARrest;
create initial var {initial};
append;
nsub=137-2;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:3]=parameters[23:25];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,25,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
data NMARsubset2;
set result;
run;
proc print data=NMARsubset2;
run;
data NMARmxlik;
set loglik;
run;
proc print data=NMARmxlik;
run;



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'
' Anslysis of subset 3
'
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


/***************************************/
/* (5a) MCAR analysis without subset 3 */
/***************************************/


data data1mnarsubset3;
set IMPI_2016;
if id in (93, 94, 33, 78, 133) then delete;
run;
proc iml;
use data1mnarsubset3;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
	y = data[,2];
create y var {y};
append;
use impi2aMCAR;
read all var {est} into mcarest;
initial=mcarest;	
create initial var {initial};
append;
nsub=137-5;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;


proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1]=parameters[23];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);

  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,23,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;

data MCARsubset3;
set result;
run;
proc print data=MCARsubset3;
run;
data MCARmxlik;
set loglik;
run;
proc print data=MCARmxlik;
run;

/***************************************/
/* (5b) MAR analysis without subset 3  */
/***************************************/

data data1marsubset2;
set IMPI_2016;
if id in (93, 94, 33, 78, 133) then delete;
run;
proc iml;
use data1marsubset2;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aMAR;
read all var{est} into MARrest;
initial=MARrest;
create initial var {initial};
append;
nsub=137-5;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:2]=parameters[23:24];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,24,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
proc print data=result;
run;
data MARsubsets2;
set result;
run;
proc print data=MARsubsets2;
run;

data MARmaxlik;
set loglik;
run;
proc print data=MARmaxlik;
run;






/*************************************/
/* (3) DIGGLE-KENWARD MODEL : MNAR   */
/*************************************/
data data2nmarsubset2;
set IMPI_2016;
if id in (93,94,33, 78,133) then delete;
run;
proc iml;
use data2nmarsubset2;
read all var {id sqcd4 transage art pred time} into data;
	id = data[,1];
	time = data[,6];
	treat = data[,5];
	timetreat =time#treat;
	art=data[,4];
	predart= treat#art;
	transage = data[,3];
	intercept=j(nrow(data),1,1);
create x var {intercept time treat timetreat art predart transage};
append;
y = data[,2];
create y var {y};
append;
use impi2aNMAR;
read all var{est} into NMARrest;
*initial=mcarest//0.5;
initial=NMARrest;
create initial var {initial};
append;
nsub=137-5;
ntime=5;
create nsub var {nsub};
append;
create ntime var {ntime};
append;
quit;

proc iml;
/*     calculating LOGLIKELIHOOD     */
use x;
read all into x;
print x;
use y;
read all into y;
use nsub;
read all into nsub;
use ntime;
read all into ntime;
use initial;
read all into initial;
g=j(nsub,1,0);

start integr(yd) global(psi,ecurr,vcurr,lastobs);
	pi = constant('PI');
	g=exp(psi[1]+psi[2]*lastobs+psi[3]*yd);
	g=g/(1+g);
	integr=g*(1/(sqrt((2*pi)*vcurr)))*exp(-(1/2)*(((yd-ecurr)**2)/vcurr));
return(integr);
finish integr;

nrun = 0;

start loglik(parameters) global(lastobs,vcurr,ecurr,x,z,y,nsub,ntime,nrun,psi);
	beta=parameters[1:7];
	u11=parameters[8];
	u12=parameters[9];
	u13=parameters[10];
	u14=parameters[11];
	u15=parameters[12];
	u22=parameters[13];
	u23=parameters[14];
	u24=parameters[15];
	u25=parameters[16];
	u33=parameters[17];
	u34=parameters[18];
	u35=parameters[19];
	u44=parameters[20];
	u45=parameters[21];
	u55=parameters[22];
	psi=j(3,1,0);
	psi[1:3]=parameters[23:25];
	ll=0;
	ind=1;
	do while (ind<=nsub);

  /* select information on one particular patient */

  	xi = x[(ind-1)*ntime+1:ind*ntime,];
  	yi = y[(ind-1)*ntime+1:ind*ntime];
	mui = xi*beta;
	vi = j(ntime,ntime,1);
	ui = j(ntime,ntime,1);
	ui[1,1] = u11;
	ui[1,2] = u12;
	ui[1,3] = u13;
	ui[1,4] = u14;
	ui[1,5] = u15;
	ui[2,1] = 0;
	ui[2,2] = u22;
	ui[2,3] = u23;
	ui[2,4] = u24;
	ui[2,5] = u25;
	ui[3,1] = 0;
	ui[3,2] = 0;
	ui[3,3] = u33;
	ui[3,4] = u34;
	ui[3,5] = u35;
	ui[4,1] = 0;
	ui[4,2] = 0;
	ui[4,3] = 0;
	ui[4,4] = u44;
	ui[4,5] = u45;
	ui[5,5] = u55;
	vi=t(ui)*ui;
	pi = constant('PI');
  	g=j(nsub,1,0);
  /* yobs & dropout */

  if (yi[nrow(yi)]=.) then   
    do;                      
		dropout=1;
		j=nrow(yi);
		do while (yi[j-1]=.);
          j=j-1;
		end;
        di=j;
        yobs=yi[1:di-1];
     end;
   else                                    
    do;                      
        dropout=0;
        yobs=yi;
    end;

  /* remove intermittent missingness   */

  sequence = t(1:nrow(yobs)); 
  miss=yobs||sequence; 
  i=1;
  do while (i<=nrow(miss));
    if (miss[i,1]=.) then do;
		nr=nrow(miss);
		row=(i-1)*2+1//(i-1)*2+2;
		miss=remove(miss,row);
		miss=shape(miss,nr-1,2);
	end;
	else 
      i=i+1;
  end;
  yobs = miss[,1];
  nummi = miss[,2];
 
  /* Calculation of loglik   */

  /* er wordt onderscheid gemaakt tussen wel of geen dropout */
  if dropout=0 then   /* COMPLETER */
    do;
		ll1=0;
		ll2=0;
        j=2;
		do while (j <= nrow(yobs));
			g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
			g = g/(g+1);
          	if (g=1) then do; 
                g=g-0.0001; 
          	end;
			ll2=ll2+log(1-g);   
			j=j+1;
		end;

            /* GET ll1 */ 
		/* yobs=yi[nummi]; */ 
        vvi=vi[nummi,nummi];
		vvii=inv(vvi);
		detvvi=det(vvi);
		muiobs=mui[nummi];
		ll1=-(ntime/2)*log(2*pi)-0.5*log(det(vvi))-0.5*t(yobs-muiobs)*vvii*(yobs-muiobs);

            /* OBTAIN ll */
        ll=ll+ll1+ll2;
    end;

	else do;          /* WITH DROPOUT */
        ll1=0;
        ll2=0;
        ll3=0;

      /* calculate ll2 */
        if (di=2) then do;
            goto skip;
		end;
        else  do;
            j=2;
			do while (j<=(nrow(yobs)));
            	g=exp(psi[1]+yobs[j-1]*psi[2]+yobs[j]*psi[3]);
            	g=g/(1+g);
            	if (g=1) then do; 
              		g=g-0.0001; 
            	end;
                ll2=ll2+log(1-g);
                j=j+1;
            end; 
		end;

        skip: 

      /* obtain ll1 */

        hist=yobs; 
      	mup=mui[nummi]; 
        vi11=vi[nummi,nummi];
      	vi11i=inv(vi11);
      	ll1=-((di-1)/2)*log(2*pi)-0.5*log(det(vi11))-0.5*t(hist-mup)*vi11i*(hist-mup); 

        /* obtain ll3 */

        muc=mui[di]; 
        vi11=vi[nummi,nummi];
        vi22=vi[di,di];
        vi12=vi[nummi,di];
        vi21=vi[di,nummi];
        ecurr=muc+vi21*inv(vi11)*(hist-mup);
        vcurr=vi22-vi21*inv(vi11)*vi12; 
        lastobs = yi[nummi[nrow(nummi)]];
        a = ecurr-2*sqrt(vcurr)||ecurr+2*sqrt(vcurr);

		call quad(integraal,"integr",a);
        ll3=integraal;
        if (ll3=0) then do; 
			ll3=ll3+0.00001; 
		end;
        ll3=log(ll3);

        /* berekening ll */
        ll=ll+ll1+ll2+ll3;
	end;
    ind=ind+1;
	end;
	loglik=ll;
	nrun=nrun+1;
	file log;
	put nrun;
return(loglik);
finish loglik;

opt=j(1,11,0);
opt[1]=1;
opt[2]=5;
con=j(2,25,.);
call nlpnrr(rc,est,"loglik",initial,opt,con);
*call nlpnms(rc,est,"loglik",initial,opt,con);
print rc;
call nlpfdd(maxlik,grad,hessian,"loglik",est);
print maxlik;
print grad;
print hessian;
inf = - hessian;
covar = inv(inf);
print covar;
var = vecdiag(covar);
print var;
stde = sqrt(var);
print stde;
print est;
esttrns=t(est);
print esttrns;
zval=esttrns/stde;
pval=2*(1-cdf('normal',abs(zval),0,1));
create result var {est var stde pval};
append;
create hessianmatrix from hessian;
append from hessian;
create covmatrix from covar;
append from covar;
create loglik from maxlik;
append from maxlik;
quit;
data NMARsubset2;
set result;
run;
proc print data=NMARsubset2;
run;
data NMARmxlik;
set loglik;
run;
proc print data=NMARmxlik;
run;
